#include <p32xxxx.h>
#include <plib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "GenericTypeDefs.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_host_hid_parser.h"
#include "USB/usb_host_hid.h"

//*****************************************************************************
//* CONFIGURATION BITS
//*****************************************************************************
    #pragma config UPLLEN   = ON            // USB PLL Enabled
    #pragma config FPLLMUL  = MUL_20        // PLL Multiplier
    #pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
    #pragma config FPLLODIV = DIV_1         // PLL Output Divider
    #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
    #pragma config FWDTEN   = OFF           // Watchdog Timer
    #pragma config WDTPS    = PS1           // Watchdog Timer Postscale
    #pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
    #pragma config OSCIOFNC = OFF           // CLKO Enable
    #pragma config POSCMOD  = HS            // Primary Oscillator
    #pragma config IESO     = OFF           // Internal/External Switch-over
    #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
    #pragma config FNOSC    = PRIPLL        // Oscillator Selection
    #pragma config CP       = OFF           // Code Protect
    #pragma config BWP      = OFF           // Boot Flash Write Protect
    #pragma config PWP      = OFF           // Program Flash Write Protect
    #pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
//*****************************************************************************
//* DATA STRUCTURES
//*****************************************************************************
typedef enum _APP_STATE
{
    DEVICE_NOT_CONNECTED,
    DEVICE_CONNECTED, 
    READY_TO_TX_RX_REPORT,
    GET_INPUT_REPORT, 
    INPUT_REPORT_PENDING,
    ERROR_REPORTED 
} APP_STATE;
//*****************************************************************************
//* Internal Function Prototypes
//*****************************************************************************
BOOL USB_ApplicationEventHandler( BYTE address, USB_EVENT event, void *data, DWORD size );
void AppInitialize(void);
void DetectPS3(void);
void App_ProcessInputReport(void);
void _mon_putc(char c);
void delayms(unsigned int ms);
void InitializeTimer( void );
void init(void);
//*****************************************************************************
// Macros
//*****************************************************************************
#define MAX_ALLOWED_CURRENT             (500)         // Maximum power we can supply in mA
#define GetSystemClock()            80000000UL
#define GetPeripheralClock()        80000000UL
//#define GetInstructionClock()       (GetSystemClock() / 2)
#define CoreTicksPerMs              (GetSystemClock() / 2000)
//*****************************************************************************
//* GLOBAL VARIABLES
//*****************************************************************************
//*****************************************************************************
APP_STATE PS3State;

BOOL DisplayConnectOnce = FALSE;
BOOL DisplayDetachOnce = FALSE;

BYTE ps3EnableCmd[] = {0x42,0x0C,0x00,0x00};
BYTE ps3ReportBuf[64];
BYTE error;
BYTE ps3LedCmd[] = {0x00, 0x96, 0x7F, 0x96, 0x7F,
                    0x00, 0x00, 0x00, 0x00, 0x02,
                    0xFF, 0x27, 0x10, 0x00, 0x32,
                    0xFF, 0x27, 0x10, 0x00, 0x32,
                    0xFF, 0x27, 0x10, 0x00, 0x32,
                    0xFF, 0x27, 0x10, 0x00, 0x32,
                    0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00
                  };
//******************************************************************************
//* USB/PS3 SUPPORT FUNCTIONS
//******************************************************************************
void delayms(unsigned int ms)
{
	unsigned int msDelayTime, currentTickCnt;
	currentTickCnt = ReadCoreTimer(); 
	msDelayTime = (CoreTicksPerMs * ms) + currentTickCnt; 
	while((ReadCoreTimer()) < msDelayTime); 
}

BOOL USB_ApplicationEventHandler( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    switch( (INT)event )
    {
        case EVENT_VBUS_REQUEST_POWER:
            // The data pointer points to a byte that represents the amount of power
            // requested in mA, divided by two.  If the device wants too much power,
            // we reject it.
            if (((USB_VBUS_POWER_EVENT_DATA*)data)->current <= (MAX_ALLOWED_CURRENT / 2))
            {
                return TRUE;
            }
            else
            {
                printf( "\r\n***** USB Error - device requires too much current *****\r\n" );
            }
            break;

        case EVENT_VBUS_RELEASE_POWER:
            // Turn off Vbus power.
            // The PIC24F with the Explorer 16 cannot turn off Vbus through software.
            return TRUE;
            break;

        case EVENT_HUB_ATTACH:
            printf( "\r\n***** USB Error - hubs are not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            printf( "\r\n***** USB Error - device is not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_CANNOT_ENUMERATE:
            printf( "\r\n***** USB Error - cannot enumerate device *****\r\n" );
            return TRUE;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            printf( "\r\n***** USB Error - client driver initialization error *****\r\n" );
            return TRUE;
            break;

        case EVENT_OUT_OF_MEMORY:
            printf( "\r\n***** USB Error - out of heap memory *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
            printf( "\r\n***** USB Error - unspecified *****\r\n" );
            return TRUE;
            break;

	case EVENT_HID_RPT_DESC_PARSED:
            return TRUE;
            break;

        default:
            break;
    }
    return FALSE;
}
//******************************************************************************
//* UART1 REDIRECTOR
//******************************************************************************
void _mon_putc(char c)
{
	U1TXREG = c;
	while(U1STAbits.TRMT == 0);
}
//******************************************************************************
//* INITIALIZE THE IN REPORT TIMER
//******************************************************************************
void InitializeTimer(void)
{
    T4CON = 0x0;			//Stop and Init Timer
    T4CON = 0x0050;			//prescaler = 1:32 - internal clock
    TMR4 = 0;				//Clear timer register
    PR4 = 0x61A8;			//Load period register - 0.010 second period

    IPC4SET = 0x00000005;	// Set priority level=1

    IFS0bits.T4IF = 0;
    IEC0bits.T4IE = 1;

    T4CONSET = 0x8000;		//Start Timer
}
//******************************************************************************
//* IN REPORT TIMER INTERRUPT HANDLER
//******************************************************************************
#pragma interrupt _T4Interrupt ipl4 vector 16
void _T4Interrupt( void )
{
    if (IFS0bits.T4IF)
    {
        IFS0bits.T4IF   = 0;
        mPORTAToggleBits(BIT_0);
        if(READY_TO_TX_RX_REPORT == PS3State)
        {
            PS3State = GET_INPUT_REPORT;
        }
    }
}
//******************************************************************************
//* APPLICATION DETECT DEVICE
//******************************************************************************
void DetectPS3(void)
{
  if(!USBHostHIDDeviceDetect(1))
  {
     PS3State = DEVICE_NOT_CONNECTED;
  }
}
//******************************************************************************
//* INITIALIZE FUNCTION
//******************************************************************************
void init(void)
 {
    int  value;

    value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );

    // Enable the cache for the best performance
    CheKseg0CacheOn();
	// Enable Interrupts
    INTEnableSystemMultiVectoredInt();
	// Wait for PLL lock to stabilize
    value = OSCCON;
    while (!(value & 0x00000020))
    {
    value = OSCCON;    
    }

     // Uncomment to check 10 mS timer
    //mJTAGPortEnable(DEBUG_JTAGPORT_OFF);
    //mPORTASetPinsDigitalOut(BIT_0);

	//Initialize UART
    setbuf(stdout,NULL);	//UART1 REDIRECT

    UARTConfigure(UART1, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetFifoMode(UART1, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UART1, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART1, GetPeripheralClock(), 115200);
    UARTEnable(UART1, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));

    PS3State = DEVICE_NOT_CONNECTED;
 }
//******************************************************************************
// MAIN
//******************************************************************************
int main (void)
{
    BYTE i;
    init();
	USBInitialize(0);
	do{
		USBTasks();

		switch(PS3State)
		{
			case DEVICE_NOT_CONNECTED:
				USBTasks();
				if(DisplayDetachOnce == FALSE)
				{
					printf("PS3 Detached\r\n");
					DisplayDetachOnce = TRUE;
				}
				// True if report descriptor is parsed with no error
				if(USBHostHIDDeviceDetect(1)) 
				{
					PS3State = DEVICE_CONNECTED;
					DisplayConnectOnce = FALSE;
				}
			break;
			case DEVICE_CONNECTED:
				PS3State = READY_TO_TX_RX_REPORT;
				if(DisplayConnectOnce == FALSE)
				{
					printf("PS3 Connected\r\n");
					DisplayConnectOnce = TRUE;
					DisplayDetachOnce = FALSE;
					USBHostIssueDeviceRequest( 0x01, 0x21, 0x09,0x03F4,0x00,0x04,ps3EnableCmd,0,0);
					delayms(6);
					USBHostIssueDeviceRequest( 0x01, 0x21, 0x09,0x201,0x00,0x30,ps3LedCmd,0,0);
				}                             
				InitializeTimer();
				break;
			case READY_TO_TX_RX_REPORT:
				if(!USBHostHIDDeviceDetect(1))
				{
					PS3State = DEVICE_NOT_CONNECTED;
				}
			break;
			case GET_INPUT_REPORT:
			error = USBHostHIDRead(0x01,0x01,0x00,0x31,ps3ReportBuf);
			for(i=0;i<0x32;++i)
			{
				printf("%0X",ps3ReportBuf[i]);
			}
			printf("\r\n");
			PS3State = READY_TO_TX_RX_REPORT;
			break;
			default:
			break;

		}
	}while(1);
}




